const Product = require('../models/productschema');
const ApiError = require('../utils/ApiError');
const httpStatus = require('../utils/httpStatus');

// upload pics
//const config = require('../config/config');
//const multer = require('multer');
//const multerS3 = require('multer-s3');
//const { S3Client } = require('@aws-sdk/client-s3');
//const { Upload } = require('@aws-sdk/lib-storage');

const createProduct = async (req, res) => {
  const product = new Product(req.body);

  // Check if the user is authenticated
  if (req.user) {
    // Set the user_id property of the product to the authenticated user's id
    product.user_id = req.user._id; // or req.user.id, depending on your user model
  } else {
    throw new ApiError(httpStatus.UNAUTHORIZED, 'Unauthorized');
  }

  const savedProduct = await product.save();
  res.status(httpStatus.CREATED).json(savedProduct);
};


/*
const getProducts = async (req, res) => {
  const products = await Product.find();
  res.json(products);
};
*/

const getProducts = async (req, res) => {
  const { page = 1, limit = 10, sortBy = 'createdAt', order = 'desc' } = req.query;

  // Validate pagination parameters
  if (isNaN(page) || isNaN(limit) || page < 1 || limit < 1) {
    throw new ApiError(httpStatus.BAD_REQUEST, 'Invalid pagination parameters');
  }

  // Validate sorting parameters
  const allowedSortFields = ['createdAt', 'title']; // Add or remove fields as needed
  if (!allowedSortFields.includes(sortBy)) {
    throw new ApiError(httpStatus.BAD_REQUEST, 'Invalid sort field');
  }

  if (!['asc', 'desc'].includes(order.toLowerCase())) {
    throw new ApiError(httpStatus.BAD_REQUEST, 'Invalid sort order');
  }

  const options = {
    page: parseInt(page, 10),
    limit: parseInt(limit, 10),
    sort: { [sortBy]: order === 'desc' ? -1 : 1 },
    lean: true,
  };

  const result = await Product.paginate({}, options);

  if (result.docs.length === 0 && page > 1) {
    throw new ApiError(httpStatus.NOT_FOUND, 'No products found for this page');
  }

  res.json({
    products: result.docs,
    totalPages: result.totalPages,
    currentPage: result.page,
    totalPosts: result.totalDocs,
  });
};

const getProduct = async (req, res) => {
  const product = await Product.findById(req.params.id);
  if (!product) {
    throw new ApiError(httpStatus.NOT_FOUND, 'Product not found');
  }
  res.json(product);
};

const getProductwithslug = async (req, res) => {
  const product = await Product.findOne({ slug: req.params.slug });
  if (!product) {
    throw new ApiError(httpStatus.NOT_FOUND, 'Product not found');
  }
  res.json(product);
};

const updateProduct = async (req, res) => {
  try {
    // Check if the user is authenticated
    if (!req.user) {
      throw new ApiError(httpStatus.UNAUTHORIZED, 'Unauthorized');
    }

    // Get the product by ID
    const product = await Product.findById(req.params.id);

    // Check if the product exists and if the authenticated user is the owner
    if (!product || product.user_id.toString() !== req.user._id.toString()) {
      throw new ApiError(httpStatus.FORBIDDEN, 'You are not authorized to update this product');
    }

    // Update the product
    const updatedProduct = await Product.findByIdAndUpdate(req.params.id, req.body, { new: true });

    res.json(updatedProduct);
  } catch (error) {
    throw new ApiError(httpStatus.INTERNAL_SERVER_ERROR, 'Something went wrong');
  }
};

const deleteProduct = async (req, res) => {
  try {
    // Check if the user is authenticated
    if (!req.user) {
      throw new ApiError(httpStatus.UNAUTHORIZED, 'Unauthorized');
    }

    // Get the product by ID
    const product = await Product.findById(req.params.id);

    // Check if the product exists and if the authenticated user is the owner
    if (!product || product.user_id.toString() !== req.user._id.toString()) {
      throw new ApiError(httpStatus.FORBIDDEN, 'You are not authorized to delete this product');
    }

    // Delete the product
    await Product.findByIdAndDelete(req.params.id);

    res.json({ message: 'Product deleted successfully' });
  } catch (error) {
    throw new ApiError(httpStatus.INTERNAL_SERVER_ERROR, 'Something went wrong');
  }
};


/* const myProducts = async (req, res) => {
  try {
    // Check if the user is authenticated
    if (!req.user) {
      throw new ApiError(httpStatus.UNAUTHORIZED, 'Unauthorized');
    }

    // Get the user ID from the authenticated user
    const userId = req.user._id;

    // Find all properties where the user_id matches the authenticated user's ID
    const products = await Product.find({ user_id: userId });

    res.json(products);
  } catch (error) {
    throw new ApiError(httpStatus.INTERNAL_SERVER_ERROR, 'Something went wrong');
  }
}; */



// image upload
// Configure AWS SDK
/* const s3Client = new S3Client({
  region: config.s3.region,
  credentials: {
    accessKeyId: config.s3.accessKeyId,
    secretAccessKey: config.s3.secretAccessKey,
  },
 });
 
 
 const upload = multer({
  storage: multerS3({
    s3: s3Client,
    bucket: config.s3.bucket,
    acl: 'public-read',
    contentType: multerS3.AUTO_CONTENT_TYPE,
    metadata: function (req, file, cb) {
      cb(null, { fieldName: file.fieldname });
    },
    key: function (req, file, cb) {
      const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1e9);
      cb(null, 'images/' + uniqueSuffix + '-' + file.originalname);
    },
  }),
 });
 
 
 const imageupload = async (req, res) => {
  const singleUpload = upload.single('image');
 
 
  singleUpload(req, res, (err) => {
    if (err) {
      // Directly send the error response if multer throws an error
      return res.status(httpStatus.BAD_REQUEST).json({
        message: `Error uploading file: ${err.message}`
      });
    }
 
 
    // If the file has been uploaded and multer-s3 has set the location, send success response
    if (req.file && req.file.location) {
      res.status(200).json({
        message: 'Image uploaded successfully',
        url: req.file.location
      });
    } else {
      // If there is no file or location, respond with an error
      res.status(httpStatus.INTERNAL_SERVER_ERROR).json({
        message: 'Failed to upload image. No file was uploaded or no location is available.'
      });
    }
  });
 }; */
 

module.exports = {
  createProduct,
  getProducts,
  getProduct,
  updateProduct,
  deleteProduct,
  getProductwithslug,
  //myProducts,
  //imageupload
};